<?php
/**
 * @file
 * Plugin for call to OpenEDU Callout blocks as panel panes.
 */

/**
 * Declare the plugin to ctools.
 */
$plugin = array(
  'single' => TRUE,
  'title' => t('OpenEDU Bean Callout'),
  'description' => '',
  'category' => t('OpenEDU Widgets'),
  'edit form' => 'openedu_bean_callout_plugin_edit_form',
  'render callback' => 'openedu_bean_callout_plugin_render',
  'admin info' => 'openedu_bean_callout_plugin_admin_info',
  'defaults' => array(
    'title' => '',
    'header' => '',
    'items' => array(),
    'footer' => '',
    'display' => ''
  )
);

/**
 * Edit form callback for the content type.
 */
function openedu_bean_callout_plugin_edit_form($form, &$form_state) {
  // Set variables.
  $options = $beans = $displays = array();
  $conf = $form_state['conf'];

  // Load all available items.
  $beans = _openedu_bean_callout_load_by_type('openedu_bean_callout');

  // Get settings from bean and transfer to form configuration to pass on to rendering stage.
  if (isset($beans['bean']) && !empty($beans['bean'])) {

    // Load all entities.
    $load_beans = _openedu_bean_callout_load(array_keys($beans['bean']));

    // Loop through them.
    foreach ($load_beans as $bean) {
      $options[$bean->bid] = $bean->title;
    }
  }

  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $conf['title'],
    '#maxlength' => 128,
  );

  $form['header_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('Header'),
    '#collapsible' => TRUE,
    '#collapsed' =>  (empty($conf['header']) ? TRUE : FALSE),
  );
  $form['header_fieldset']['header'] = array(
    '#type' => 'textarea',
    '#title' => t('Content'),
    '#default_value' => $conf['header'],
  );

  $form['items_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['items_fieldset']['items'] = array(
    '#type' => 'select',
    '#multiple' => TRUE,
    '#title' => t('Select all items to be display at once'),
    '#options' => $options,
    '#default_value' => array_values($conf['items']),
    '#required' => TRUE,
  );

  $form['footer_fieldset'] = array(
    '#type' => 'fieldset',
    '#title' => t('Footer'),
    '#collapsible' => TRUE,
    '#collapsed' => (empty($conf['footer']) ? TRUE : FALSE),
  );
  $form['footer_fieldset']['footer'] = array(
    '#type' => 'textarea',
    '#title' => t('Content'),
    '#default_value' => $conf['footer'],
  );

  // Try loading view to display all displays.
  if (($view = views_get_view('openedu_bean_callout')) == TRUE &&
    isset($view->display)) {
    foreach ($view->display as $display_id => $display_info) {
      if (strtolower($display_id) !== 'default') {
        $displays[$display_id] = $display_info->display_title;
      }
    }
  }

  $form['display'] = array(
    '#type' => 'select',
    '#title' => t('View mode'),
    '#options' => $displays,
    '#default_value' => $conf['display'],
  );

  $form['#attached']['js'] = array(
    drupal_get_path('module', 'openedu_bean_callout') . '/libraries/multi-select/js/jquery.multi-select.js',
    drupal_get_path('module', 'openedu_bean_callout') . '/libraries/quicksearch/jquery.quicksearch.js',
    drupal_get_path('module', 'openedu_bean_callout') . '/js/openedu_bean_callout.admin.js'
  );

  $form['#attached']['css'] = array(
    drupal_get_path('module', 'openedu_bean_callout') . '/libraries/multi-select/css/multi-select.css'
  );

  return $form;
}

/**
 * The submit form stores the data in $conf.
 */
function openedu_bean_callout_plugin_edit_form_submit($form, &$form_state) {
  foreach (array_keys($form_state['plugin']['defaults']) as $key) {
    if (isset($form_state['values'][$key])) {
      $form_state['conf'][$key] = $form_state['values'][$key];
    }
  }
}

/**
 * Run-time rendering of the body of the block (content type).
 * See ctools_plugin_examples for more advanced info.
 */
function openedu_bean_callout_plugin_render($subtype, $conf, $panel_args, $context = NULL) {
  // Set vars.
  $block = new stdClass();
  $bean_content = NULL;
  $display = '';

  // Check if variables are available.
  if (isset($conf['display']) && isset($conf['items'])) {
    $display = $conf['display'];
    $bean_content = views_embed_view('openedu_bean_callout', $display, implode(',', array_values($conf['items'])));
  }

  // Set title and content.
  $block->title = '';
  $block->content = theme('openedu_bean_callout', array(
    'title'   => (!empty($conf['title']) ? $conf['title'] : FALSE),
    'header'  => (!empty($conf['header']) ? $conf['header'] : FALSE),
    'content' => $bean_content,
    'footer'  => (!empty($conf['footer']) ? $conf['footer'] : FALSE),
    'class'   => 'openedu-bean-callout-' . $display,
    'id'      => ''
  ));

  return $block;
}

/**
 * Admin info callback for panel pane.
 */
function openedu_bean_callout_plugin_admin_info($subtype, $conf, $contexts) {
  if (!empty($conf)) {
    // Set values.
    $display = $xitems = '';
    $block = new stdClass;
    $block->title = $conf['override_title'] ? $conf['override_title_text'] : '';

    if (!empty($conf['items']) && ($items = _openedu_bean_callout_load(array_values($conf['items']))) == TRUE) {
      foreach ($items as $item) {
        $title = $item->title;

        $xitems .= '<li>'. $title .'</li>';
      }
      $display = '<ul>'. $xitems .'</ul>';
    }
    else {
      $display = t('Sorry, no items were selected.');
    }

    $block->content = t('Items being displayed: !items', array(
      '!items' => $display,
    ));

    return $block;
  }
}
