<?php

/**
 * @file
 * Views hooks implemented for the OpenEDU Notifications module.
 */

/**
 * Implements hook_views_data().
 */
function openedu_notifications_views_data() {
  // This is a OpenEDU Notifications Server only functionality.
  if (!defined('OPENEDU_NOTIFICATIONS_SERVER_TABLE_NOTIFICATIONS')) {
    return;
  }

  // Setup table array.
  $data = array();

  // Sets the views group of the new table.
  $data[OPENEDU_NOTIFICATIONS_SERVER_TABLE_NOTIFICATIONS]['table']['group'] = t('OpenEDU Notifications');

  // Sets a title to the new data set.
  $data[OPENEDU_NOTIFICATIONS_SERVER_TABLE_NOTIFICATIONS]['table']['base'] = array(
    'title' => t('OpenEDU Notifications'),
    'field' => 'id',
    'weight' => 10,
    'help' => t('Contains notification records from OpenEDU Notifications.'),
  );

  // Exposes the fields to Views.
  $data[OPENEDU_NOTIFICATIONS_SERVER_TABLE_NOTIFICATIONS]['id'] = array(
    'title' => t('Notification ID'),
    'help' => t('This holds the unique indentifier for the OpenEDU Notifications system records.'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
     ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );

  $data[OPENEDU_NOTIFICATIONS_SERVER_TABLE_NOTIFICATIONS]['uid'] = array(
    'title' => t('The user id.'),
    'help' => t('The user associated with notification.'),
    'field' => array(
      'handler' => 'views_handler_field_numeric',
      'click sortable' => TRUE,
     ),
    'filter' => array(
      'handler' => 'views_handler_filter_numeric',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );

  $data[OPENEDU_NOTIFICATIONS_SERVER_TABLE_NOTIFICATIONS]['data'] = array(
    'title' => t('Notification Body'),
    'help' => t('The content of the notification.'),
    'field' => array(
      'handler' => 'views_handler_field_serialized',
      'click sortable' => TRUE,
     ),
    'filter' => array(
      'handler' => 'views_handler_field_serialized',
    ),
    'sort' => array(
      'handler' => 'views_handler_sort',
    ),
  );

    // Exposes notification 'read status' to views.
    $data[OPENEDU_NOTIFICATIONS_SERVER_TABLE_NOTIFICATIONS]['status'] = array(
        'title' => t('OpenEDU Notification Read Status'),
        'help' => t('OpenEDU Notification Read Status'),
        'field' => array(
            'handler' => 'views_handler_field_numeric',
            'click sortable' => TRUE,
        ),
        'filter' => array(
          'handler' => 'views_handler_filter_numeric',
        ),
        'sort' => array(
          'handler' => 'views_handler_sort',
        ),
    );

  return $data;
}
