(function ($) {
  /**
   * Adds behaviour layout to OpenEDU Dashboard Schools.
   */
  Drupal.behaviors.OpenEDUSchoolsDashboard = {
    setMessage: function(message, type) {
      // Set vars.
      var main_content = $('body').find('#content'),
          message_wrapper = main_content.find('#message-wrapper');

      // Check if a message container is already present.
      if (!message_wrapper.length) {
        main_content.prepend($('<div />').attr({
          id: 'message-wrapper'
        }));
      }

      // Append message.
      main_content.find('#message-wrapper').html($('<div />').attr({
        'class': 'messages ' + type
      }).text(message).delay(5000).slideUp('slow'));
    },
    saveData: function(ui) {
      // Set vars.
      var callbackURL = Drupal.settings.openeduSchoolsDashboard.updatePositionCallbackURL,
          data = {
            table: '',
            entity_id: '',
            bundle: '',
            items: ''
          },
          closest = ui.item.closest('ul'),
          elements = (closest.length) ? closest.find('> li') : false;

      // Check if URL is available.
      if (!callbackURL) {
        Drupal.behaviors.OpenEDUSchoolsDashboard.setMessage(Drupal.t('No callback URL available.'), 'error');
        return false;
      }

      if (elements.length) {
        // Load all data attr.
        elements.each(function(k, v) {
          var thisData = $(this).data();

          data.table = thisData.table;
          data.entity_id = thisData.entity_id;
          data.bundle = thisData.bundle;
          data.items = data.items +  thisData.target_id +',';
        });

        // Submit info against server.
        $.post( Drupal.settings.basePath + callbackURL, {
          table: data.table,
          entity_id: data.entity_id,
          bundle: data.bundle,
          items: data.items
        })
        .done(function(result) {
            Drupal.behaviors.OpenEDUSchoolsDashboard.done(result);
        })
        .fail(function(xhr) {
            Drupal.behaviors.OpenEDUSchoolsDashboard.fail(xhr);
        });
      }
      else {
        Drupal.behaviors.OpenEDUSchoolsDashboard.setMessage(Drupal.t('Sorry, no data has been submitted.'), 'error');
      }
    },
    done: function(result) {
      Drupal.behaviors.OpenEDUSchoolsDashboard.setMessage(Drupal.t('Data saved!'), 'status');
    },
    fail: function(xhr) {
      Drupal.behaviors.OpenEDUSchoolsDashboard.setMessage(Drupal.t('Request failed, check your console for more information.'), 'error');
    },
    attach: function(context, settings) {
      /**
       * Set click behaviour.
       */
      var schools = $('.view-openedu-schools-dashboard', context);

      if (schools.length) {
        schools.find('h2.dept .title').on('click', function(){
          var $this = $(this),
            dpt_details = $this.parents('.views-row').find('.dept-details');

          // Check if department structure is present.
          if (dpt_details.length) {
            dpt_details.slideToggle();
          }
        });
      }

      /**
       * Toggle all button.
       */
      var toggle_all = $('a.toggle-all-departments', context);

      if (toggle_all.length) {
        toggle_all.on('click', function(e) {
          e.preventDefault();
          $(this).parents('.quicktabs-tabpage').find('.dept-details').slideToggle();
        });
      }

      /**
       * Sortable.
       */
      var sortable_selectors = {
        // Department listing
        0: $('ul.openedu-schools-dashboard-department-list'),
        // Dept Program Listing
        1: $('ul.openedu-schools-dashboard-dept-program-list'),
        // Course Listing
        2: $('ul.openedu-schools-dashboard-course-list'),
        // Dept Course Listing
        3: $('ul.openedu-schools-dashboard-dpto-course-list')
      };

      // Enable sort.
      $.each(sortable_selectors, function(k, v) {
        if (v.length) {
          v.sortable({
            update: function(event, ui) {
              Drupal.behaviors.OpenEDUSchoolsDashboard.saveData(ui);
            }
          }).disableSelection();
        }
      });
    }
  };
})(jQuery);
