<?php
/**
 * @file
 * Plugin for call to action blocks as panel panes.
 */

/**
 * Declare the plugin to ctools.
 */
$plugin = array(
  'single' => TRUE,
  'title' => t('OpenEDU Syndication Bean'),
  'description' => 'Allows to show content shared via Syndication',
  'category' => t('OpenEDU Syndication'),
  'edit form' => 'openedu_syndication_bean_syndication_edit_form',
  'render callback' => 'openedu_syndication_bean_syndication_render',
  'admin info' => 'openedu_syndication_bean_admin_info',
  'defaults' => array(
    'bid' => NULL,
    'limit' => '',
    'sort_by' => '',
    'sort_order' => '',
  )
);

/**
 * Edit form callback for the content type.
 */
function openedu_syndication_bean_syndication_edit_form($form, &$form_state) {
  // Fetch all existing Syndication beans.
  $beans = _openedu_syndication_bean_load_by_type('syndication');

  // Set options array.
  $options = array();

  // Get settings from bean and transfer to form configuration to pass on to rendering stage.
  if (isset($beans['bean']) && !empty($beans['bean'])) {

    // Load all entities.
    $load_beans = _openedu_syndication_bean_beans_load(array_keys($beans['bean']));

    // Loop through them.
    foreach ($load_beans as $bean) {
      $options[$bean->bid] = $bean->title;
    }
  }

  // Create the field to select the bean.
  $form['bid'] = array(
    '#title' => 'Please select an existing bean to display',
    '#type' => 'select',
    '#default_value' => $form_state['conf']['bid'],
    '#options' => $options,
    '#required' => TRUE,
  );

  // Create the fieldset to override the bean defaults.
  $form['bean_overrides'] = array(
    '#type' => 'fieldset',
    '#title' => 'Bean Overrides',
  );

  // Add the field overrides.
  $form['bean_overrides']['limit'] = array(
    '#type' => 'select',
    '#title' => t('Number of items'),
    '#default_value' => ($form_state['conf']['limit']) ?
      $form_state['conf']['limit'] : 0,
    '#options' => array(
      '' => 'No override',
      5 => '5',
      10 => '10',
      15 => '15'
    ),
  );
  $form['bean_overrides']['sort_by'] = array(
    '#type' => 'select',
    '#title' => t('Sort key'),
    '#default_value' => ($form_state['conf']['sort_by']) ?
      $form_state['conf']['sort_by'] : 0,
    '#options' => array(
      '' => 'No override',
      'title' => 'Title',
      'created' => 'Date',
    ),
  );
  $form['bean_overrides']['sort_order'] = array(
    '#type' => 'select',
    '#title' => t('Sort Order'),
    '#default_value' => ($form_state['conf']['sort_order']) ?
      $form_state['conf']['sort_order'] : 0,
    '#options' => array(
      '' => 'No override',
      'ASC' => 'Ascending',
      'DESC' => 'Descending',
    ),
  );
  $form['bean_overrides']['custom_css_class'] = array(
    '#type' => 'textfield',
    '#title' => t('Custom CSS Class'),
    '#default_value' => ($form_state['conf']['custom_css_class']) ?
      $form_state['conf']['custom_css_class'] : '',
  );

  return $form;
}

/**
 * The submit form stores the data in $conf.
 */
function openedu_syndication_bean_syndication_edit_form_submit($form, &$form_state) {
  foreach (array_keys($form_state['plugin']['defaults']) as $key) {
    if (isset($form_state['values'][$key])) {
      $form_state['conf'][$key] = $form_state['values'][$key];
    }
  }
}

/**
 * Run-time rendering of the body of the bean (Syndication).
 * See ctools_plugin_examples for more advanced info.
 */
function openedu_syndication_bean_syndication_render($subtype, $conf, $panel_args, $context = NULL) {
  // Set vars.
  $block = new stdClass();
  $data = $data_fields = '';

  // Get bean block info.
  $result = _openedu_syndication_bean_beans_load(array($conf['bid']));
  $bean = $result[$conf['bid']];

  // CSS class build-out.
  $bean_custom_class = (isset($bean->field_custom_css_class[LANGUAGE_NONE][0]['value'])) ?
    ' ' . $bean->field_custom_css_class[LANGUAGE_NONE][0]['value'] : '';
  $panelizer_custom_class = (!empty($conf['custom_css_class'])) ?
    ' ' . $conf['custom_css_class'] : '';
  $classes = 'openedu-syndication-client-render' . $bean_custom_class . $panelizer_custom_class;

  // Set data to be appended to the wrapper.
  $data_fields = array(
    'endpoint' => isset($bean->field_available_endpoints[LANGUAGE_NONE][0]['value']) ?
      $bean->field_available_endpoints[LANGUAGE_NONE][0]['value'] : FALSE,

    'limit' => (int)$conf['limit'] > 0 ?
      $conf['limit'] : $bean->field_limit[LANGUAGE_NONE][0]['value'],

    'sort_by' => !empty($conf['sort_by']) ?
      $conf['sort_by'] : $bean->field_sort_by[LANGUAGE_NONE][0]['value'],

    'sort_order' => !empty($conf['sort_order']) ?
      $conf['sort_order'] : $bean->field_sort_order[LANGUAGE_NONE][0]['value'],
  );
  foreach ($data_fields as $k => $v) {
    $data .= 'data-'. $k .'="'. $v .'" ';
  }

  // Set/Configure title.
  $block->title = !empty($conf['override_title_text']) ? $conf['override_title_text'] : '';

  // Set/Configure body.
  $block->content  = '<div class="syndication-bean">' . $bean->title . '</div>';
  $block->content .= '<div id="bean-' . $conf['bid'] . '" class="' . $classes . '" ' . $data . '></div>';

  return $block;
}

/**
 * Admin info callback for panel pane.
 */
function openedu_syndication_bean_admin_info($subtype, $conf, $contexts) {
  if (!empty($conf['bid'])) {
    // Set values.
    $display = '';
    $block = new stdClass;

    // Allow overridden title.
    $block->title = $conf['override_title'] ? $conf['override_title_text'] : '';

    // Attempt to load bean.
    if($result = _openedu_syndication_bean_beans_load(array($conf['bid']))) {
      $bean = $result[$conf['bid']];
      $display = '';

      // Populate bean specifications.
      if($endpoint = $bean->field_available_endpoints[LANGUAGE_NONE][0]['value']) {
        $display .= '<div>Endpoint: ' . $endpoint . '</div>';
      }

      // Check for number of items overrides.
      $item_limit = (!empty($conf['limit'])) ?
        $conf['limit'] : $bean->field_limit[LANGUAGE_NONE][0]['value'];
      $display .= '<div>Number of items: ' . $item_limit . '</div>';

      // Check for sort key overrides.
      $sort_by = (!empty($conf['sort_by']) ?
        $conf['sort_by'] : $bean->field_sort_by[LANGUAGE_NONE][0]['value']);
      $display .= '<div>Ordered by: ' . $sort_by . '</div>';

      // Check for sort order overrides.
      $sort_order = (!empty($conf['sort_order']) ?
        $conf['sort_order'] : $bean->field_sort_order[LANGUAGE_NONE][0]['value']);
      $display .= '<div>Sort Order: ' . $sort_order . '</div>';

      // Check for custom class overrides.
      $custom_class = (!empty($conf['custom_class']) ?
        $conf['custom_class'] : (isset($bean->field_custom_css_class[LANGUAGE_NONE][0]['value']) ?
          $bean->field_custom_css_class[LANGUAGE_NONE][0]['value'] : ''));
      $display .= '<div>Custom CSS Class: ' . $custom_class . '</div>';
    }

    // Adds a title to the specifications.
    $block->content = $display;

    return $block;
  }
}