<?php

/**
 * Register Syndication Server form.
 */
function openedu_syndication_global_form($form, &$form_state) {
  $form = array();

  // Endpoint validation.
  $form['endpoint'] = array(
    '#type' => 'fieldset',
    '#title' => t('Endpoint validation'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE
  );
  $form['endpoint']['openedu_syndication_endpoint_validation'] = array(
    '#title' => t('Enabled'),
    '#type' => 'checkbox',
    '#default_value' => openedu_syndication_endpoint_validation(),
    '#description' => t('Disabling this option will allow user to add/edit endpoints without any validation.')
  );

  // Ctools plugin.
  $form['ctools'] = array(
    '#type' => 'fieldset',
    '#title' => t('Ctools plugin'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE
  );
  $form['ctools']['openedu_syndication_ctools_plugin'] = array(
    '#title' => t('Enabled'),
    '#type' => 'checkbox',
    '#default_value' => _openedu_syndication_ctools_plugin(),
    '#description' => t('Enabling <i>!ctools</i>', array(
      '!ctools' => l('Ctools', 'https://www.drupal.org/project/ctools')
    ))
  );

  // Enable/Disable content types be syndicated.
  $form['content'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content types enabled for sharing.'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  // Check if content types are available.
  if (($node_types = node_type_get_types()) == TRUE) {
    $options = array();

    foreach ($node_types as $type => $info) {
      $options[$type] = $info->name;
    }

    // Display all options so users can set which ones should be shared.
    $form['content'][OPENEDU_SYNDICATION_VAR_CONTENT_TYPES] = array(
      '#type' => 'checkboxes',
      '#title' => t('Select all content types that should be globally enabled for Syndication.'),
      '#options' => $options,
      '#default_value' => variable_get(OPENEDU_SYNDICATION_VAR_CONTENT_TYPES, array())
    );
  }

  // Enforce validate and submit.
  $form['#submit'][] = 'openedu_syndication_global_form_submit';

  return system_settings_form($form);
}

/**
 * Register Syndication Server form submit.
 */
function openedu_syndication_global_form_submit($form, &$form_state) {
  // Load vars.
  $values = $form_state['values'];

  // Save variables.
  variable_set('openedu_syndication_endpoint_validation' ,$values['openedu_syndication_endpoint_validation']);
  variable_set('openedu_syndication_ctools_plugin', $values['openedu_syndication_ctools_plugin']);

  // Save content types allowed to be shared.
  variable_set(OPENEDU_SYNDICATION_VAR_CONTENT_TYPES, array_filter($form_state['values'][OPENEDU_SYNDICATION_VAR_CONTENT_TYPES]));

  // Set feedback.
  drupal_set_message(t('The configuration options have been saved.'));

  // Redirect user.
  drupal_goto(OPENEDU_SYNDICATION_ADMIN_PATH . '/global');
}
